////////////////////////////////////////////////////////////////////////
#include<AnisEvent.hh>
#include<Steer.hh>
////////////////////////////////////////////////////////////////////////

extern Steer *steerdata;
extern IsNeutrino is_neutrino;

////////////////////////////////////////////////////////////////////////
 AnisWeightContainer::AnisWeightContainer(){
    FluxWeight=0;
    AtmosEWeight=0;
    AtmosMWeight=0;
    FinalVertexWeight=0;
 };

////////////////////////////////////////////////////////////////////////
//constructor for AnisEvent is used from HepMC::GenEvent
 AnisEvent::AnisEvent(int ievent) : HepMC::GenEvent(1,ievent){
    FinalVolume.NewIntersection();
 }

////////////////////////////////////////////////////////////////////////
 void AnisEvent::print_event( std::ostream& ostr ) {

   //first shift all vertex positions into detector coordinates

   for ( GenEvent::vertex_const_iterator vtx = this->vertices_begin();
	 vtx != this->vertices_end(); ++vtx ) {
     Hep3Vector position((*vtx)->position().x(),(*vtx)->position().y(),
       (*vtx)->position().z());
     HepLorentzVector p_d(position-steerdata->DetectorCenter,
			  (*vtx)->position().t());
     (*vtx)->set_position(p_d);
   }

   switch (steerdata->Format) {
   case 1:      // HepMC format
       this->print();
       break;
   case 2:      // f2k(AMANDA) format
       this->print_f2k();
       break;
   case 3:      // pheno format
       this->print_pheno();
       break;
   default:
       cerr << "unknown format: steerdata->Format !!" << endl;
   }
 }

////////////////////////////////////////////////////////////////////////
 void AnisEvent::print_f2k( std::ostream& ostr ) {
//   if you want to write this event to file outfile.txt, please use:
//   std::ofstream outfile("outfile.txt"); event.print( outfile );

   Printf2k printf2k;   
// event header
   ostr << " EM " << this->event_number() << " 1 ? ? 0.000000000 0" << endl; 

   int    nuinitial_flavor = 0;  // dummy initialization, will be overwritten
   double nuinitial_energy = 0;  // dummy initialization, will be overwritten
   int    nufinal_flavor   = 0;  // init to zero, somethime not filled later!
   double nufinal_energy   = 0;
   double nuinitial_barcode = 0;  // necessary to keep CLHEP from choking


   Hep3Vector direction =  this->FinalVolume.GetUnitDirection();
   double theta = direction.getTheta()*180./PI;  
   theta = 180-theta;
   double phi = 180+direction.getPhi()*180./PI;  
   double ns = 1e9;               // unit: s->ns

   int particle_number = 2;  //initializing counter; 1 is reserved for initial neutino
// loop over all Vertices to find the final ones
   for ( GenEvent::vertex_const_iterator vtx = this->vertices_begin();
  	vtx != this->vertices_end(); ++vtx ) {
         Hep3Vector p_d((*vtx)->position().x(),(*vtx)->position().y(),
       		 (*vtx)->position().z());
        Hep3Vector position = position+steerdata->DetectorCenter;
// find inital neutrino
	if (vtx == this->vertices_begin()) {
	  for ( HepMC::GenVertex::particle_iterator p = 
		  (*vtx)->particles_begin(HepMC::children);
		p != (*vtx)->particles_end(HepMC::children); ++p) {
	    ostr <<" TR " << "1 0 "<< printf2k.pdg_to_name((*p)->pdg_id())  
                << " " << p_d.x() << " " << p_d.y() << " " << p_d.z() << " " 
                << theta << " " << phi << " 0 " << (*p)->momentum().e() << " "
                << (*vtx)->position().t()*ns << endl ;
	   nuinitial_energy=(*p)->momentum().e();
	   nuinitial_flavor=(*p)->pdg_id();
	   nuinitial_barcode=(*vtx)->barcode();
	  }
	}	
   }


   for ( GenEvent::vertex_const_iterator vtx = this->vertices_begin();
	 vtx != this->vertices_end(); ++vtx ) {
     Hep3Vector p_d((*vtx)->position().x(),(*vtx)->position().y(),
		    (*vtx)->position().z());
     Hep3Vector position = position+steerdata->DetectorCenter;
     
       if (FinalVolume.IsInVolume(position)
	   && (*vtx)->barcode()!=nuinitial_barcode) {
//information of entering neutrino 
	   for ( HepMC::GenVertex::particle_iterator p = 
	        (*vtx)->particles_begin(HepMC::parents);
	        p != (*vtx)->particles_end(HepMC::parents); ++p) {
	     if (nufinal_flavor == 0) { //this is only true for first nu in vol
	       nufinal_energy = (*p)->momentum().e();
	       nufinal_flavor = (*p)->pdg_id();
	     }
	   }

// write out all final particles       

	   for ( HepMC::GenVertex::particle_iterator p = 
	         (*vtx)->particles_begin(HepMC::children);
	       p != (*vtx)->particles_end(HepMC::children); ++p) {
//  if ( !(*p)->end_vertex() && (*p)->status()==1) {
	     if ( (*p)->status()==1) {

	       // momentum defined direction; this will be important for 
	       // implementing final lepton angle.
	       //double theta=acos((*p)->momentum().z()/(*p)->momentum().e());
	       double theta=(*p)->momentum().theta();
	       theta=180.-180./PI*theta;
	       //double phi=atan2((*p)->momentum().y(),(*p)->momentum().x());
	       double phi=(*p)->momentum().phi();
	       phi=180.+180./PI*phi;
	       
	       double parent_nu=1;
	       ostr << " TR "<<particle_number<<" "<<parent_nu<<" "
		    << printf2k.pdg_to_name((*p)->pdg_id())<<" "<<p_d.x()<<" "
		    << p_d.y() <<" "<<p_d.z()<<" "<<theta<<" "<<phi<<" 0 "
		    << (*p)->momentum().e()<<" "<<(*vtx)->position().t()*ns<< endl ;
	       //   cout<<"final:"<<theta<<" "<<phi<<" "<<p_d.x()<<" " <<p_d.y()<<" "<<p_d.z()<<endl;
	       particle_number++;
	     }
	   }
       }
   }   
   ostr << " US wght " << this->EventWeight.GetFinalVertexWeight() << " "
	<< this->EventWeight.GetFluxWeight() << " "
	<< this->EventWeight.GetAtmosphericEWeight()<< " "
	<< this->EventWeight.GetAtmosphericMWeight() << endl;
   ostr << " US info " <<nuinitial_flavor << " " << nufinal_flavor<< " " 
	<< nuinitial_energy << " " << nufinal_energy << " " << endl;
   ostr << " EE" << endl;
 }
////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////
// print_pheno is used to write out a few more variables  
////////////////////////////////////////////////////////////////////////
 void AnisEvent::print_pheno( std::ostream& ostr ) {
//   if you want to write this event to file outfile.txt, please use:
//   std::ofstream outfile("outfile.txt"); event.print( outfile );


   int    nuinitial_flavor = 0;  // dummy initialization, will be overwritten
   double nuinitial_energy = 0;  // dummy initialization, will be overwritten
   int    nufinal_flavor   = 0;  // init to zero, somethime not filled later!
   double nufinal_energy   = 0;

   double x1=0;  //first vertex coordinates
   double y1=0;
   double z1=0;
   double fout1=0; 
   double eout1=0; 
   double ehad1=0;
   double x2=0;  //possibly second vertex
   double y2=0;
   double z2=0;
   double eout2=0; 
   double fout2=0;
   double ehad2=0;

   Hep3Vector direction =  this->FinalVolume.GetUnitDirection();
   double theta = direction.getTheta();  
   
// loop over all Vertices to find the final ones
   for ( GenEvent::vertex_const_iterator vtx = this->vertices_begin();
  	vtx != this->vertices_end(); ++vtx ) {
         Hep3Vector p_d((*vtx)->position().x(),(*vtx)->position().y(),
       		 (*vtx)->position().z());
        Hep3Vector position = position+steerdata->DetectorCenter;
// find inital neutrino
       if (vtx == this->vertices_begin()) {
         for ( HepMC::GenVertex::particle_iterator p = 
	       (*vtx)->particles_begin(HepMC::children);
	     p != (*vtx)->particles_end(HepMC::children); ++p) {
	     nuinitial_energy=(*p)->momentum().e();
	     nuinitial_flavor=(*p)->pdg_id();
         }
       }

// see if the vertex is inside

       if (FinalVolume.IsInVolume(position)) {

//obtaining information of entering neutrino 

	   for ( HepMC::GenVertex::particle_iterator p = 
	        (*vtx)->particles_begin(HepMC::parents);
	        p != (*vtx)->particles_end(HepMC::parents); ++p) {
	       //find the incoming neutrino
	       if (is_neutrino(*p) && nufinal_flavor == 0) { 
//this is only true for first nu in vol
		   nufinal_energy = (*p)->momentum().e();
		   nufinal_flavor = (*p)->pdg_id();
		   x1=p_d.x();
		   y1=p_d.y();
		   z1=p_d.z();
		   // check final states
		   for ( HepMC::GenVertex::particle_iterator f_p = 
			     (*vtx)->particles_begin(HepMC::children);
			 f_p != (*vtx)->particles_end(HepMC::children); ++f_p){
		       
		       if ( (*f_p)->status()==1) {
			   if (abs((*f_p)->pdg_id())==11 || 
			       abs((*f_p)->pdg_id())==13 || 
			       abs((*f_p)->pdg_id())==15) { 
			       eout1= (*f_p)->momentum().e();
			       fout1= (*f_p)->pdg_id();
			   }
			   if (abs((*f_p)->pdg_id())==211) //its a hadron
			       ehad1= (*f_p)->momentum().e();
			   
			   if (abs((*f_p)->pdg_id())==15){  //its a tau

			       Hep3Vector decay_position(
				   (*f_p)->end_vertex()->position().x(),
				   (*f_p)->end_vertex()->position().y(), 
				   (*f_p)->end_vertex()->position().z());
			       Hep3Vector p_decay = decay_position
				   -steerdata->DetectorCenter;
			       for ( HepMC::GenVertex::particle_iterator s_p
					 = (*f_p)->end_vertex()->
					 particles_begin(HepMC::children); 
				     s_p != (*f_p)->end_vertex()->
					 particles_end(HepMC::children); ++s_p){
				   if (abs((*s_p)->pdg_id())==11 || abs((*s_p)->pdg_id())==13 || abs((*s_p)->pdg_id())==15) { 
				       eout2= (*s_p)->momentum().e();
				       fout2= (*s_p)->pdg_id();
				   }
				   if (abs((*s_p)->pdg_id())==211) { //its a hadron
				       ehad2= (*s_p)->momentum().e();
				   }
				   x2=p_decay.x();
				   y2=p_decay.y();
				   z2=p_decay.z();
			       }

			   }
			   
		       }
		       
		   }
	       }
	   }
       }
   }
   
 
   ostr<< this->EventWeight.GetFinalVertexWeight() << " "  
       << this->EventWeight.GetFluxWeight() << " "
       << this->EventWeight.GetAtmosphericEWeight() << " " 
       << this->EventWeight.GetAtmosphericMWeight() << " " 
       << nuinitial_flavor << " " << nuinitial_energy << " " <<  theta 
       << " " << nufinal_flavor << " " << nufinal_energy << " " 
       <<x1<<" "<<y1<<" "<<z1<<"  "<<eout1<<" "<<fout1<<"  "<<ehad1<<" "
       <<x2<<" "<<y2<<" "<<z2<<"  "<<eout2<<" "<<fout2<<"  "<<ehad2<<endl;
   
 }
////////////////////////////////////////////////////////////////////////









