////////////////////////////////////////////////////////////////////////
// AnisEvent contains the specialized event class  used in ANIS
// It is derived from the HepMC event and has additional 
// geometry related member(s)
////////////////////////////////////////////////////////////////////////

#ifndef ANISEVENT 
#define ANISEVENT 
#include <AnisGlobal.hh>
#include <AnisParticles.hh>
#include <AnisIO.hh>
#include <FinalVolume.hh>
////////////////////////////////////////////////////////////////////////
// this is the weight container of the anis event class. It currently 
// stores the amtospheric flux weight and the interaction weight of 
// the final vertex
////////////////////////////////////////////////////////////////////////
 class AnisWeightContainer {
   public:
    AnisWeightContainer();
    double GetFluxWeight()                     {return FluxWeight       ;} 
    void SetFluxWeight(double fluxwght)        {FluxWeight=fluxwght     ;} 
    double GetAtmosphericEWeight()     {return AtmosEWeight      ;} 
    double GetAtmosphericMWeight()     {return AtmosMWeight      ;} 
    void SetAtmosphericEWeight(double atmoswght){AtmosEWeight=atmoswght;} 
    void SetAtmosphericMWeight(double atmoswght){AtmosMWeight=atmoswght   ;} 
    double GetFinalVertexWeight()              {return FinalVertexWeight;} 
    void SetFinalVertexWeight(double weight)   {FinalVertexWeight=weight;} 
   private:
    double FluxWeight;
    double AtmosMWeight;  // atmospheric muon neutrino weight 
    double AtmosEWeight; // atmospheric electron neutrino weight
    double FinalVertexWeight;
 };

////////////////////////////////////////////////////////////////////////
// This is the AnisEvent class. It is derived from HepMC::GenEvent 
////////////////////////////////////////////////////////////////////////
 class AnisEvent : public HepMC::GenEvent {
  public:
   AnisEvent(int ievent);
   FinalVolumeCylinder FinalVolume;
   void print_event( std::ostream& ostr = std::cout ); // dumps  event to ostr 
   AnisWeightContainer EventWeight;   
   bool DidFinalInteracion(); 

  private:
   void print_f2k( std::ostream& ostr = std::cout ); //dumps f2k event to ostr
   void print_pheno( std::ostream& ostr = std::cout ); // dumps f2k event to ost
 };

////////////////////////////////////////////////////////////////////////
// Assistant class
// this predicate returns true if the input has no decay vertex
 class IsFinalState {
  public:
    bool operator()( const HepMC::GenParticle* p ) {
        if ( !p->end_vertex() && p->status()==1 ) return 1;
        return 0;
    }
 };
////////////////////////////////////////////////////////////////////////
#endif





