////////////////////////////////////////////////////////////////////////
// Implementation of the class Earth
////////////////////////////////////////////////////////////////////////
#include <Earth.hh>

////////////////////////////////////////////////////////////////////////
//  Earth::Earth(){ }
////////////////////////////////////////////////////////////////////////
// Returns the density as a function of 3D-Vector
////////////////////////////////////////////////////////////////////////
   double Earth::GetDensity(Hep3Vector R){
    double rr = R.mag();
    return GetDensity(rr);
  }

////////////////////////////////////////////////////////////////////////
//  Distribution of the matter density in the Earth measured in g/cm^3
//  as the function of distance from the center R measured in meters
//  Earth model was referred in Gandhi et al.  astro-ph/9512364 p.23
////////////////////////////////////////////////////////////////////////
    double Earth::GetDensity(double R_meters){
     double dens = 0;
     double x = R_meters/EARTHRADIUS;
       if (R_meters < 0) {
        cerr << "  Attention, R < 0 !!! " << endl;
        exit(EXIT_FAILURE); 
       }
       else if (R_meters < 1221500) dens = 13.0885 - 8.8381*x*x;
       else if (R_meters < 3480000) dens = 12.5815 - x*(1.2638 + x*(3.6426 + x*5.5281));
       else if (R_meters < 5701000) dens = 7.9565 - x*(6.4761 - x*(5.5283 - x*3.0807));
       else if (R_meters < 5771000) dens = 5.3197 - 1.4836*x;
       else if (R_meters < 5971000) dens = 11.2494 - 8.0298*x;
       else if (R_meters < 6151000) dens = 7.1089 - 3.8045*x;
       else if (R_meters < 6346600) dens = 2.691 + 0.6924*x;
       else if (R_meters < 6356000) dens = 2.9;
       else if (R_meters < (EARTHRADIUS - 3000) ) dens = 2.6;
       else if (R_meters <= EARTHRADIUS) dens = 0.93;
       else if (R_meters <= ATMORADIUS)  dens = 0;
       else if (R_meters > ATMORADIUS){
       cerr << " Attention, R = " << R_meters 
            << " is higher than ATMOSRADIUS =" << ATMORADIUS 
            << " m !!! " << endl;
       cerr << " R - ATMORADIUS = " << R_meters - ATMORADIUS 
            << " m" << endl;
       exit(EXIT_FAILURE);
       }
     return dens;
   }
////////////////////////////////////////////////////////////////////////




