////////////////////////////////////////////////////////////////////////
#include <FinalVertex.hh>
////////////////////////////////////////////////////////////////////////

 extern SigmaTotal *sig_tot;
 extern IsNeutrino is_neutrino;
 extern IsUnStable is_unstable;
 extern Earth earth; 

////////////////////////////////////////////////////////////////////////
 FinalVertex::FinalVertex(){}

////////////////////////////////////////////////////////////////////////
float LeptonRange(int pdg_id, float E) {

  float range=0.;
  float loge=-1e10;
  if (E>0) loge=log10(E);
  switch (abs(pdg_id)) {
  case 11:
    // electron:
    range=0.;
    break;
  case 13:
    // muon:
    // rough parametrization of muon range (mwe):
    // maximum muon length: multiply range with 1.3
    range=1.3*3e3*log(1+E/6e2);
    if (range>1e5) range=1e5;
    break;
  case 15:
    // tau: range calculation to be added
    range=0.;
    break;
  default:
    cerr<<"LeptonRange(): Unknown lepton flavor "<<pdg_id
	<<endl;
    return 0.;
  }
  return range;
}

////////////////////////////////////////////////////////////////////////
 void FinalVertex::ThrowVertex(AnisEvent* evt){
   //    TauDecay decay;  
    double material[] = {0.5,0.5,0.55555}; // {p,n,e} assume H_2O, might be changed
    evt->EventWeight.SetFinalVertexWeight(0.); 
    extern Steer *steerdata;
    // loop over all particles and look for
    // - charged leptons which can reach the detector
    // - neutrinos which did not interact
    // If a charge lepton is found, the FinalVolume is increased to
    // include this lepton, the interaction weight is set to 1 and
    // no additional vertex is created.
    // Otherwise, one of the neutrinos is selected to create a vertex
    // in a volume extended to the lepton range.

    Hep3Vector position = evt->FinalVolume.GetExitIntersection();
    Hep3Vector direction = evt->FinalVolume.GetUnitDirection();

    float finallength=evt->FinalVolume.GetLength();
    const int MAXNU=100;
    float prob[MAXNU], sigma[MAXNU], range[MAXNU];
    float maxrange=0.;
    HepMC::GenParticle *neutrinos[MAXNU];
    int reactingneutrino=0;
    int selectednu=-1;
    int extravertex=1;
    // length in [mwe] of the final volume as defined in the steering file:
    for ( GenEvent::particle_const_iterator p = evt->particles_begin();
	  p != evt->particles_end(); ++p ) {
      float lepton_range, add_range;
    //don't do the final vertex if neutrino is behind the cylinder
      if (evt->FinalVolume.IsBehindVolume((*p)->production_vertex()
					  ->position().vect()))
	continue;

      switch ( abs((*p)->pdg_id()) ) {
      case 11:
      case 13:
      case 15:
	// check if the lepton can reach the FinalVolume
	if (steerdata->ExtendedVolume)
	  add_range=LeptonRange((*p)->pdg_id(), (*p)->momentum().e())
	    /earth.GetDensity(EARTHRADIUS-100)-finallength;
	// (a more precise test could be done using the integrated
	// column density, to reduce the acceptance volume to ~1/3)
	else
	  add_range=0.;
	// the lepton interaction can take place at least in the final volume
	// defined by HBCyl and HACyl:
	

	
	
	if (add_range<0.)
	  add_range=0.;
	if (((*p)->production_vertex()->position().vect()
	     -evt->FinalVolume.GetExitIntersection()).mag()
	    <add_range+finallength) {
	  GenVertex::particle_iterator parentnu;
	  for (parentnu = (*p)->production_vertex()
		 ->particles_begin(HepMC::parents);
	       parentnu != (*p)->production_vertex()
		 ->particles_end(HepMC::parents);
	       ++parentnu)
	    if (is_neutrino(*parentnu)) {
	      if (reactingneutrino<MAXNU) {
		float sig=sig_tot->GetSigma(*parentnu,material);
		range[reactingneutrino]=add_range+finallength;
		sigma[reactingneutrino]=sig;
		neutrinos[reactingneutrino]=*parentnu;
		// this neutrino will be the one used for weight calculation:
		selectednu=reactingneutrino;
		reactingneutrino++;
		extravertex=0;
	      } else {
		cerr<<"FinalVertex::ThrowVertex warning: More than "<<MAXNU<<
		  " neutrinos\n";
	      }
	      break;
	    }
	}
	break;
      case 12:
      case 14:
      case 16:
	if( !(*p)->end_vertex() ) {
	  if (reactingneutrino<MAXNU) {
	    int lepton_id=abs((*p)->pdg_id())-1;
	    if (steerdata->ExtendedVolume)
	      lepton_range=LeptonRange(lepton_id, (*p)->momentum().e());
	    else
	      lepton_range=finallength;
	    if (lepton_range<finallength)
	      lepton_range=finallength;
	    float sig=sig_tot->GetSigma(*p,material);
	    range[reactingneutrino]=lepton_range;
	    sigma[reactingneutrino]=sig;
	    neutrinos[reactingneutrino]=*p;
	    reactingneutrino++;
	  } else {
	    cerr<<"FinalVertex::ThrowVertex warning: More than "<<MAXNU<<
	      " neutrinos\n";
	  }
	}
      }      
    }
    if (reactingneutrino<=0) {
      evt->EventWeight.SetFinalVertexWeight(0.); 
      return;
    }

    // correct the lepton ranges if the volumina are not contained in the
    // earth or go back further than the neutrino production vertex
    // If no extra vertex is needed, this correction is skipped
    if (extravertex) {
      maxrange=0.;
      for (int nu=0; nu<reactingneutrino; nu++) {
	if ((position-range[nu]*direction).mag()>EARTHRADIUS) {
	  range[nu]=0;
	  if (position.mag()<EARTHRADIUS){
	    float tmp=position*direction;
	    range[nu]=tmp+sqrt(tmp*tmp+EARTHRADIUS*EARTHRADIUS-position.mag2());	  }
	}
	float tmp=(position-neutrinos[nu]->production_vertex()
		   ->position().vect())*direction;
	if (range[nu]>tmp)
	  range[nu]=tmp;
	if (range[reactingneutrino]>maxrange)
	  maxrange=range[reactingneutrino];
      }
    }
    
    // calculate the column depths:
    float dx=maxrange/1000.;
    if (dx<1.) dx=1.;
    float length=0.;
    int remaining=reactingneutrino;
    float depth;
    for (int nu=0; nu<reactingneutrino; nu++)
      prob[nu]=-1.;
    for (depth=0.; remaining>0; ) {
      float dx_=dx;
      if (dx_>length/100.) // avoid too large steps in the beginning
	dx_=length/100.;
      if (dx_<1.) dx_=1.;

      depth+=dx_*earth.GetDensity(position);
      length+=dx_;
      position-=dx_*direction;
      for (int nu=0; nu<reactingneutrino; nu++)
	if (prob[nu]<0.)
	  if (length>=range[nu]) {
	    prob[nu]=depth;
	    --remaining;
	  }
      // if we know already which neutrino will be used, the loop can be
      // stopped when its column depth has been calculated:
      if (!extravertex && prob[selectednu]>=0.)
	remaining=0;
    }

    float sumprob=0.;
    if (extravertex) {
      // depth -> "probabilities"
      for (int nu=0; nu<reactingneutrino; nu++) {
	sumprob+=prob[nu]*sigma[nu];
	prob[nu]=sumprob;
      }

      float rand_prob=drand48() * sumprob;
      for (selectednu=0; selectednu<reactingneutrino; selectednu++)
	if (prob[selectednu]>=rand_prob)
	  break;
      if (selectednu>=reactingneutrino)
	cerr<<"Error in ThrowVertex: No neutrino selected.\n"
	    <<"(THIS SHOULD NOT HAPPEN)\n";
    } else {
      sumprob=prob[selectednu]*sigma[selectednu];
    }
    
    // Now that we decided which neutrino interacts, the final volume
    // is extended according to the range of the possibly produced
    // charged lepton:
    evt->FinalVolume.ExtendBackward(range[selectednu]-finallength);

    if (extravertex) {
      Hep3Vector interaction = Position(sigma[selectednu], evt->FinalVolume); 
      GenVertex *vtx = new GenVertex (HepLorentzVector(interaction,0),1,0); 
      
      vtx->add_particle_in(neutrinos[selectednu]);
      sig_tot->FillVertex(vtx,material);
      evt->add_vertex(vtx);
      // add unstable particles created in vtx:
      for ( HepMC::GenVertex::particle_iterator pout = 
	      vtx->particles_begin(HepMC::children);
	    pout != vtx->particles_end(HepMC::children); ++pout) {
	if (is_unstable(*pout)&& steerdata->FinalDecay){ 
	  decay.AddVertex(*pout,evt);
	}		
      }
    }
    
    // calculating the weight of the vertex
    // wght = Sum{sigma * Mvolume} / Mproton / Area 
    // sum of all cross sections:  sumsigma  is different 
    // from sigma only in case of more than one neutrino
    double norm =   1.0e-25 / PMASS;  
    
    double wght = norm * sumprob;
    // calculating interaction probability
    wght = 1-exp(-wght);
    // filling the ANIS weight
    evt->EventWeight.SetFinalVertexWeight(wght); 

    // filling the HepMC wght
    evt->weights().push_back(wght);
    // and the flux weights
    evt->weights().push_back(evt->EventWeight.GetFluxWeight());
    evt->weights().push_back(evt->EventWeight.GetAtmosphericEWeight());
    evt->weights().push_back(evt->EventWeight.GetAtmosphericMWeight());
 }

////////////////////////////////////////////////////////////////////////
 Hep3Vector FinalVertex::Position(double sigma, FinalVolumeCylinder finalvolume){

// position in final final volume is chosen according to matter depth 
// distribution. No regeneration in final volume!


// first the interaction probability is calculated:
     
   double wght = sigma*ColumnDepth(finalvolume)*1.0e-25/PMASS; 
   double rand_probability = drand48()*(1-exp(-wght)); 



// init the while loop
   Hep3Vector localposition = finalvolume.GetIntersection();
   double dx    = finalvolume.GetLength()/imax;
   double depth = 0;
   double probability =0;
// choose interaction bin
   while (probability<rand_probability){
      localposition += finalvolume.GetUnitDirection( )*dx;
      depth         += earth.GetDensity(localposition)*dx;
      wght =  sigma * depth * 1.0e-25/PMASS; 
      probability = 1-exp(-wght);  //probability for interaction in depth
   }
// add a random fraction of a dx bin
    localposition   += finalvolume.GetUnitDirection()*dx*drand48();

    return localposition;
 }  

////////////////////////////////////////////////////////////////////////
 double FinalVertex::ColumnDepth(FinalVolumeCylinder finalvolume){
   Hep3Vector localposition = finalvolume.GetIntersection();
    double dx = finalvolume.GetLength()/imax;
    double depth = 0;
    for (int i=1;i<imax;i++){
      localposition += finalvolume.GetUnitDirection( )*dx;
      depth         += earth.GetDensity(localposition)*dx;
    }
    return depth;
 }  
////////////////////////////////////////////////////////////////////////

EventSelector::EventSelector(int usewght){
    if (usewght!=0) 
	UseWght=1;
    else 
	UseWght=0;

    if (!UseWght) {
	maxwght=FindMaxwght();
    }
}
bool EventSelector::IsSelected(AnisEvent* evt){
    bool useevent;
    double y_rand=drand48()*maxwght;
    if (UseWght) {
	useevent=1;
    } else {
	if (evt->EventWeight.GetFinalVertexWeight()>y_rand) {
	    useevent=1;
	    if (evt->EventWeight.GetFinalVertexWeight()>maxwght) {
		evt->EventWeight.SetFinalVertexWeight(
		    evt->EventWeight.GetFinalVertexWeight()/maxwght);
		
		cerr<<"warrning: maxwght is smaller than event weight" <<endl;
		cerr<<"setting event weight to weight/maxweight (instead of 1)" <<endl;
	    } else {
		evt->EventWeight.SetFinalVertexWeight(1.);
	    }
	} else {
	    useevent=0;
	}
    }
    return useevent;
}

double EventSelector::FindMaxwght(){
    int max=1000;
    double maxvector[3];

    FluxDriver flux;   // Flux generator
    FinalVertex final;
    Propagate prop;

// initialize the highest weights
    int nlist=10;
    double* wghtlist;
    wghtlist = new double[nlist];
    for (int i=0; i<nlist;i++)
	wghtlist[i]=0;
    // now sampleing the weights
    for (int ievent=1; ievent <= max; ievent++){
	AnisEvent* evt = new AnisEvent(ievent);
	flux.Inject(evt);  //inject neutrino  into event
	prop.ThroughEarth(evt);
	final.ThrowVertex(evt);
	double wght=evt->EventWeight.GetFinalVertexWeight();
	int j=nlist-1;
	if (wght>wghtlist[j]){
	    while (wght>wghtlist[j])
		j--;
	    for (int i=nlist-2;i>j;i--)
		wghtlist[i+1]=wghtlist[i];
	    
	    wghtlist[j+1]=wght;
	}
	delete evt;
    }
    // the maximal weight is the maximal weight found + the difference 
    // between max and 10 before max
    maxwght=2*wghtlist[0]-wghtlist[nlist];

    return maxwght;
}






