////////////////////////////////////////////////////////////////////////
#include<FinalVolume.hh>
#include<Steer.hh>
////////////////////////////////////////////////////////////////////////
 extern Steer *steerdata;
////////////////////////////////////////////////////////////////////////
 FinalVolumeCylinder::FinalVolumeCylinder(){
    SetRmax      (steerdata->Rmax      );
    SetHBCylinder(steerdata->HBCylinder);   
    SetHACylinder(steerdata->HACylinder);   
    SetCosmin    (steerdata->FluxCosmin);
    SetCosmax    (steerdata->FluxCosmax);
    XYZDetector = steerdata->DetectorCenter;
    Init = 0;
 }

////////////////////////////////////////////////////////////////////////
// Next the intersection coordinates as well as the direction
// of the neutrino (=cylinder) are randomly thrown
  void FinalVolumeCylinder::NewIntersection(){
     XYZInter.setZ(HBCylinder);     // z-coordinate in local (xyzcenter=0) coord
     double x = Rmax;
     double y = Rmax;
     while (sqrt(x*x+y*y) > Rmax){  // x and y coordinates
	 x = 2*(drand48() - 0.5)*Rmax;
	 y = 2*(drand48() - 0.5)*Rmax;
     }
     XYZInter.setX(x);
     XYZInter.setY(y);
// throw direction of cylinder
     CosTheta = drand48()*(Cosmax-Cosmin) + Cosmin;
     Phi      = drand48()*2*PI;

// rotation of XYZInter is done by hand

     double Theta =acos(CosTheta);
     Hep3Vector unitdirection(0,0,1);
     unitdirection.setTheta(Theta);
     unitdirection.setPhi(Phi);

     XYZInter=XYZInter.rotateUz(unitdirection);

//shift into global coordinates
     XYZInter += XYZDetector;
 
  }

////////////////////////////////////////////////////////////////////////
 Hep3Vector FinalVolumeCylinder::GetIntersection(){
     return XYZInter;
 }

////////////////////////////////////////////////////////////////////////
 Hep3Vector FinalVolumeCylinder::GetExitIntersection(){
     return XYZInter+GetLength()*GetUnitDirection();
 }

////////////////////////////////////////////////////////////////////////
void FinalVolumeCylinder::ExtendBackward(double length) {
  HBCylinder+=length;
  XYZInter-=length*GetUnitDirection();
}

////////////////////////////////////////////////////////////////////////
 Hep3Vector FinalVolumeCylinder::GetUnitDirection(){
    double sintheta = sin(acos(CosTheta));
    Hep3Vector direction(cos(Phi)*sintheta, sin(Phi)*sintheta, CosTheta);
    return -1*direction;
 }

////////////////////////////////////////////////////////////////////////
 int FinalVolumeCylinder::IsInVolume(Hep3Vector coordinate){
   Hep3Vector diff = (coordinate-XYZInter)*GetUnitDirection();
   int inside = 0;
   if ( diff > 0 && diff < (HBCylinder + HACylinder) )
     inside = 1;
   return inside;
 }

////////////////////////////////////////////////////////////////////////
int FinalVolumeCylinder::IsBehindVolume(Hep3Vector coordinate){
    int behind=0;
    if ((coordinate-GetExitIntersection())*GetUnitDirection()>0){
	behind=1;
    }

    return behind;

}
////////////////////////////////////////////////////////////////////////













