////////////////////////////////////////////////////////////////////////
// FinalVolume includes all the geometric properties related to the final 
// volume. It also contains the intersection points for the neutrino track.
// MK 11.11.02
////////////////////////////////////////////////////////////////////////
#include <AnisGlobal.hh>
#include <CLHEP/Vector/ThreeVector.h>

#ifndef FINALVOLUME
#define FINALVOLUME

////////////////////////////////////////////////////////////////////////
 class FinalVolumeCylinder {
   public:
      FinalVolumeCylinder();
      Hep3Vector GetIntersection ();  //this is the intersection on final volume
      Hep3Vector GetExitIntersection ();  // point where the neutrino track leaves the final volume again
      Hep3Vector GetUnitDirection();  //this is the direction vector of the volume 
      void       NewIntersection ();
      int    IsInVolume(Hep3Vector coordinate);
      int    IsBehindVolume(Hep3Vector coordinate);

      void   SetRmax      (double rho ){Rmax       = rho ;}
      void   SetHBCylinder(double hcyl){HBCylinder = hcyl;}
      void   SetHACylinder(double hcyl){HACylinder = hcyl;}
      void   SetCosmin    (double cs  ){Cosmin     = cs  ;}
      void   SetCosmax    (double cs  ){Cosmax     = cs  ;}
      double GetPhi    () {return Phi                    ;} 
      double GetTheta  () {return acos(CosTheta)         ;}
      double GetGenArea() {return Rmax*Rmax*PI           ;}
      double GetLength () {return HBCylinder+HACylinder  ;}
      void ExtendBackward(double length);
   private:
      double Rmax;             // radius of cylinder 
      double HBCylinder;       // height of cylinder BEFORE detector plane
      double HACylinder;       // height of cylinder AFTER detector plane
      Hep3Vector XYZDetector;  // center of cylinder
      double Cosmax  , Cosmin;
      double CosTheta, Phi   ;
      Hep3Vector XYZInter; // x,y,z  of intersection of track with nu;
      int Init;
 };

#endif




