////////////////////////////////////////////////////////////////////////
#include <FluxDriver.hh>
#include <Steer.hh>
////////////////////////////////////////////////////////////////////////
  extern Steer *steerdata;
////////////////////////////////////////////////////////////////////////
  atsfluxarray::atsfluxarray() {
     ifstream mu_infile;
     ifstream e_infile;
//  data  tables 
     char atmo_e [1000];
     char atmo_mu[1000];

     char *path = steerdata->DataDir;
     strcpy(atmo_e,path);
     strcpy(atmo_mu,path);
     strcat(atmo_e ,"/atmos/atmos_e.dat");
     strcat(atmo_mu,"/atmos/atmos_mu.dat");

     mu_infile.open(atmo_mu);
     e_infile.open (atmo_e );
   
     for(i=0; i<=11; i++) {
       for(j=0; j<=8; j++) {
         mu_infile >> mu_array[i][j];
         e_infile  >> e_array [i][j];
//    cout << i << " " << j << " " <<  array[i][j] << endl;
       }
     }
     for(i=0; i<=11; i++) {
       for(j=1; j<=8; j++) {
         mu_array[i][j] = mu_array[i][j]*pow(10.0,mu_array[i][0]);
         e_array [i][j] = e_array [i][j]*pow(10.0,e_array [i][0]);
//     cout << i << " " << j << " " <<  array[i][j] << endl;
       }
     }    
// next read in the nu to nu_bar ratio table.........    
     for(i=0; i<=11; i++) {
       for(j=0; j<=7; j++) {
         mu_infile >> mu_ratio[i][j];
         e_infile  >> e_ratio [i][j];
//  cout << i << " " << j << " " <<  ratio[i][j] << endl;
       }
     }
     mu_infile.close();
     e_infile.close ();
  }

////////////////////////////////////////////////////////////////////////
  atmos_nu::atmos_nu() { 
     int i,j;
     for(i=0; i<=11; i++) {
      for(j=1; j<=8; j++) {
	fa.mu_array[i][j] = fa.mu_ratio[i][j-1] / 
           ( 1.0 + fa.mu_ratio[i][j-1] ) * fa.mu_array[i][j];
	fa.e_array [i][j] = fa.e_ratio [i][j-1] / 
           ( 1.0 + fa.e_ratio [i][j-1] ) * fa.e_array [i][j];
//      cout << i << " " << j << " " <<  fa.array[i][j] << endl;
      }
    }
  } 

////////////////////////////////////////////////////////////////////////
  atmos_nubar::atmos_nubar() { 
      int i,j;
// cout << " constructor for atmos_nubar " << endl;     
// cout << fa.array[0][0] << endl;
      for(i=0; i<=11; i++) {
        for(j=1; j<=8; j++) {
          fa.mu_array[i][j] = 1.0 / 
            ( 1.0 + fa.mu_ratio[i][j-1] ) * fa.mu_array[i][j];
          fa.e_array [i][j] = 1.0 / 
            ( 1.0 + fa.e_ratio [i][j-1] ) * fa.e_array [i][j];
// cout << i << " " << j << " " <<  fa.array[i][j] << endl;
        }
      }
  }



////////////////////////////////////////////////////////////////////////
  GammaFlux::GammaFlux(){
    SetEmin  (steerdata->FluxEmin      );
    SetEmax  (steerdata->FluxEmax      );
    SetCosmin(steerdata->FluxCosmin    );
    SetCosmax(steerdata->FluxCosmax    );
    SetNevent(steerdata->NumberOfEvents);
    SetGamma(steerdata->FluxGamma);
    LogEmin=log10(Emin);
    LogEmax=log10(Emax);
  }

////////////////////////////////////////////////////////////////////////
  double GammaFlux::ThrowEnergy(){
      double energy;
      double rand_y=drand48();
      if (Gamma==1) {
	  double logenergy = rand_y*(LogEmax-LogEmin)+LogEmin;        
	  energy= pow(10,logenergy);
      } else if (Gamma>0) {
	  energy = (1-rand_y)*pow(Emin,1-Gamma)+rand_y*pow(Emax,1-Gamma);
          energy=pow(energy,1/(1-Gamma));
      } else if (Gamma<0) {
	  cerr << "warning: FluxGamma is smaller than 0. This is not forseen!"<<endl;
      } else {
	  cerr << "warning: FluxGamma in steering file s somehow unreal"<<endl;
      }

      return energy;
  }

////////////////////////////////////////////////////////////////////////
  double GammaFlux::GetFlux(double energy){
      double flux=pow(energy,-1.*Gamma);
      return flux;
  }

////////////////////////////////////////////////////////////////////////
  double GammaFlux::GetNorm(){
      double norm;
      double time=31536000;  // seconds per year

      if (Gamma==1) 
	  norm = time*(LogEmax-LogEmin)*LN10;
      else 
	  norm = time*1/(Gamma-1)*(pow(Emin,1-Gamma)-pow(Emax,1-Gamma));

      norm*=(Cosmax-Cosmin)*2*PI; //space angle 
      norm*=1e4; //units :cm^2 -> m^2
      return norm;
  }

////////////////////////////////////////////////////////////////////////
  FluxDriver::FluxDriver(){
    SetRmax         (steerdata->Rmax         );
    SetInitialFlavor(steerdata->InitialFlavor);
    
    if (!strcmp(steerdata->FluxType,"p")){
      GammaFlux* fluxtmp = new GammaFlux;
      TheFlux=fluxtmp;
    } else if (0) { }// add new flux generators here
          
  }

////////////////////////////////////////////////////////////////////////
  void FluxDriver::Inject(AnisEvent* evt){
// get the Intersection point of the neutrino on the sens. volume
    Hep3Vector inter     =  evt->FinalVolume.GetIntersection ();
    Hep3Vector direction =  evt->FinalVolume.GetUnitDirection();
    PrimaryCoord.SetCoordinate(inter, direction);
// create vertex
    GenVertex* vertex = new GenVertex(
	   HepLorentzVector(PrimaryCoord.GetVertex(),0),1,1);    
//  neutrino flavor
    int flavor=GetInitialFlavor();
//is it an anti neutrino?
    if (drand48() < 0.5) 
	flavor *= -1;
//  neutrino energy
    double energy = TheFlux->ThrowEnergy();
//  create neutrino (  momentum = energy*direction )
    GenParticle* nuout = new GenParticle( 
	HepLorentzVector(energy*direction,energy),flavor,1);    
//  add neutrino to the event
    vertex->add_particle_out(nuout);
    evt->add_vertex(vertex);
// add weights to the event
// first the electron neutrino flux
    double atmoewght = AtmoFlux.GetFlux(energy,PrimaryCoord.GetLocalTheta()
					,flavor/abs(flavor)*12);
// second the muon neutrino flux    
    double atmomwght = AtmoFlux.GetFlux(energy,PrimaryCoord.GetLocalTheta()
					,flavor/abs(flavor)*14);
    atmoewght /= TheFlux->GetFlux(energy); //since the simu. flux follows gamma
    atmomwght /= TheFlux->GetFlux(energy); //since the simu. flux follows gamma
    evt->EventWeight.SetAtmosphericEWeight(atmoewght);
    evt->EventWeight.SetAtmosphericMWeight(atmomwght);
    
    evt->EventWeight.SetFluxWeight(TheFlux->GetNorm()*
		     evt->FinalVolume.GetGenArea()/steerdata->NumberOfEvents); 
  }

////////////////////////////////////////////////////////////////////////  
  void PrimaryCoordinate::SetCoordinate (
    Hep3Vector finalintersection, Hep3Vector unitdirection) {
    NeutrinoTrajectory traject(finalintersection,unitdirection);
    double tmin = traject.getTaumin(    );
    XYZprimary  = traject.R_Tau    (tmin);

// check theta
    Theta = unitdirection.getTheta();
    Phi   = unitdirection.getPhi  ();
    double detector_center = finalintersection.getR();
    if (detector_center > EARTHRADIUS){ // it should not be like that
       detector_center = EARTHRADIUS;
    }
    LocalTheta=asin(sin(Theta)*detector_center/EARTHRADIUS);
  }

////////////////////////////////////////////////////////////////////////
  double AtmosphericFlux::GetFlux(double energy, double theta, int flavor){
      double      flux     ;
      atmos_nu    nuflux   ;
      atmos_nubar nubarflux;
      if (abs(flavor)==14 || abs(flavor)==12){
         if (flavor > 0)
	    flux = nuflux.show_flux(energy,cos(theta),flavor);
	 else
	    flux = nubarflux.show_flux(energy,cos(theta),flavor);
      } 
      else {
	flux = 0.;
      }
    flux *= 2; // since one specifies flux for (nu+nubar) !  
    return flux;
  }

////////////////////////////////////////////////////////////////////////
  double atmos::show_flux(double energy, double cosangle, int flavor) {
     int ehi,elo;
     int alo=1; //dummy initalization
     int ahi=2; //dummy initalization
     double cosvals[9] = { 0.0, 1.0, 0.6, 0.4, 0.3, 0.2, 0.1, 0.05, 0.0 };
     double eval;
     double x00,x10,x01,x11,f1,f2,intval;
     if(cosangle <  0.0 ) { 
       cosangle = -1*(cosangle);
     }
     if((energy > 1.0) && (energy < pow(10.0,5.5))) {
       if((cosangle <= 1.0)  && (cosangle >  0.6)) { alo = 1; ahi = 2; }
       if((cosangle <= 0.6)  && (cosangle >  0.4)) { alo = 2; ahi = 3; }
       if((cosangle <= 0.4)  && (cosangle >  0.3)) { alo = 3; ahi = 4; }
       if((cosangle <= 0.3)  && (cosangle >  0.2)) { alo = 4; ahi = 5; }
       if((cosangle <= 0.2)  && (cosangle >  0.1)) { alo = 5; ahi = 6; }
       if((cosangle <= 0.1)  && (cosangle > 0.05)) { alo = 6; ahi = 7; }
       if((cosangle <= 0.05) && (cosangle >= 0.0)) { alo = 7; ahi = 8; }
       if(1.0 < cosangle <  0.0 ) { 
        cerr << " cosangle out of range !! " << endl; 
        cerr << " exit -1 " << endl; 
        exit(-1); 
       }
       elo = int(2.0*log10(energy)); 
       ehi = elo + 1; eval = 2.0*log10(energy);
       if (abs(flavor)==14){
	 x00 = log10(fa.mu_array[elo][alo]);
	 x10 = log10(fa.mu_array[ehi][alo]);
// This statement causes a seg fault when reading in files with 
// cosangle < 0 
// a patch as above meight influence flux calculations....
	x01 = log10(fa.mu_array[elo][ahi]);
	x11 = log10(fa.mu_array[ehi][ahi]);
      } else if (abs(flavor)==12){
	x00 = log10(fa.e_array[elo][alo]);
	x10 = log10(fa.e_array[ehi][alo]);
	x01 = log10(fa.e_array[elo][ahi]);
	x11 = log10(fa.e_array[ehi][ahi]);
      } else {
	  x00 = 0;
	  x01 = 0;
	  x10 = 0;
	  x11 = 0;
      }	  
      f1 = pow(10.0,(eval-elo)*(x10-x00) + x00);
      f2 = pow(10.0,(eval-elo)*(x11-x01) + x01);
      intval = f1 +(f2-f1)*(cosangle-cosvals[alo])/(cosvals[ahi]-cosvals[alo]);
    }
    else { 
      intval = 0.0; 
    }
    if (!(intval>0)){
      intval=0.0;
    }
    return intval;
}
////////////////////////////////////////////////////////////////////////












