////////////////////////////////////////////////////////////////////////
// Fluxdriver contains the classes to add neutrinos according to a 
// desired flux to an event  
////////////////////////////////////////////////////////////////////////

#ifndef FLUXDRIVER
#define FLUXDRIVER

#include <AnisParticles.hh>
#include <AnisEvent.hh>
#include <AnisGlobal.hh>
#include <NeutrTraject.hh>

////////////////////////////////////////////////////////////////////////
// class holding the atmospheric neutrino flux data
 class AtmosphericFlux { 
   public:
     double GetFlux(double energy, double theta, int flavor);
 };

////////////////////////////////////////////////////////////////////////
 class atsfluxarray {
  public:
    double mu_array[12][9];
    double mu_ratio[12][8];
    double e_array[12][9];
    double e_ratio[12][8];
    int    i,j;
//  void fluxarray(void); //class to read in flux tables 
//  variable flavor: 1=mu 2=elec
    atsfluxarray();
 }; 
// end atsfluxarray 

////////////////////////////////////////////////////////////////////////
//class to show interpolated flux
 class atmos : atsfluxarray {
   public:
   atsfluxarray fa;
   atmos(){}
   double show_flux(double energy, double cosangle, int flavor);
 };
// end atmos

////////////////////////////////////////////////////////////////////////
 class atmos_nu : public atmos {
   public:
    atmos_nu();
 };
// end atmos_nu

////////////////////////////////////////////////////////////////////////
 class atmos_nubar : public atmos {
   public:
     atmos_nubar();
 };
// end atmos_nubar

// end atmospheric flux section

// this is the base class for all Flux implementations.
// in case you want to add a new one, look at GammaFlux how it is done.
class Flux { 
public:
  virtual double GetFlux(double energy)=0;
  virtual double ThrowEnergy()=0;
  virtual double GetNorm()=0;
};

////////////////////////////////////////////////////////////////////////
 class GammaFlux : public Flux { 
   public:
     GammaFlux();
     double GetFlux(double energy);
     double ThrowEnergy();
     double GetNorm();  
     void   SetEmin  (double energy){Emin = energy;}
     void   SetEmax  (double energy){Emax = energy;}
     void   SetCosmin(double cos   ){Cosmin  = cos          ;}
     void   SetCosmax(double cos   ){Cosmax  = cos          ;}
     void   SetGamma (double gamma  ){Gamma  = gamma        ;}
     void   SetNevent(int nevent   ){Nevent  = nevent       ;}
   private:
     double LogEmin;
     double LogEmax;
     double Emin;
     double Emax;
     double Cosmin;
     double Cosmax;
     double Gamma;
     int    Nevent;
 }; 
// end GammaFlux

////////////////////////////////////////////////////////////////////////
// class for calculating and storing the primary interaction coordinates
 class PrimaryCoordinate {
   public:
     PrimaryCoordinate(){}
     void       SetCoordinate(Hep3Vector FinalInters, Hep3Vector Direction);
     Hep3Vector GetVertex    (){return XYZprimary;}
     double     GetTheta     (){return Theta     ;}
     double     GetPhi       (){return Phi       ;}
     double     GetLocalTheta(){return LocalTheta;}
   private:
     Hep3Vector XYZprimary;
     double     Theta;
     double     LocalTheta;
     double     Phi;
 };
// end PrimaryCoordinate

////////////////////////////////////////////////////////////////////////
 class FluxDriver {
   public:
     FluxDriver();
     void   Inject(AnisEvent* evt);
     void   SetRmax         (double rho ){Rmax          = rho ;}
     void   SetHCylinder    (double hcyl){HCylinder     = hcyl;}
     void   SetInitialFlavor(int    flav){InitialFlavor = flav;}
     double GetGenArea      (           ){return Rmax*Rmax*PI ;}
     int    GetInitialFlavor(           ){return InitialFlavor;}
   private:
     int    InitialFlavor;
     double Rmax;
     double HCylinder;
     PrimaryCoordinate PrimaryCoord;
     AtmosphericFlux AtmoFlux;
     Flux* TheFlux;
 };
// end FluxDriver

#endif 

