////////////////////////////////////////////////////////////////////////
// class NeutrTraject
////////////////////////////////////////////////////////////////////////
#include "NeutrTraject.hh"

////////////////////////////////////////////////////////////////////////
//  Initialization of the  NeutrinoTrajectory 
//  Input: 3D-vectors Rc and D
//  Rc gives the position of the neutrino trajectory on the cylinder,
//  D is an arbitrary 3D-vector. It defines the direction of the neutrino trajectory
//  Global variable ATMORADIUS

////////////////////////////////////////////////////////////////////////
 NeutrinoTrajectory::NeutrinoTrajectory(Hep3Vector Rc, Hep3Vector D){
   double b, snang, bcos, bsng, root;
// slightly decrease the ATMORADIUS to avoid the round off effects
// in solution of the quadratic equation (just 1 m less) !!
   const double ATM = ATMORADIUS - 1.0; 
   Rcyl  = Rc;                  
   rn    = Rcyl/ATM;         // normalization of Rc
   dn    = D.unit();         // make unite vector of the neutrino direction
   angle = rn.angle(dn);     // an angle between Rc and D  vectors
   snang = sin(angle);
   b     = rn.mag ();        // abs(rn)
   if ( (1 - b)*b < 0 ) {
     cerr <<  " Sorry, this point on cylynder is out of Atmosphere !!!" << endl;
     cerr << " b = " << b << endl;
     exit(EXIT_FAILURE);
   }  

// To find the x_min/max, we solve the quadratic equation
// x^2 + 2*bcos*x + b^2 - 1 = 0
// x_{1/2} = -bcos +/- sqrt(1-(bsin)^2) = -bcos +/- sqrt(1-bsin)*sqrt(1+bsin)

   bsng       = b*snang;
   bcos       = b*cos(angle);
   root       = sqrt (1-bsng)*sqrt(1+bsng); 
   tau_max    = -bcos + root ;
   tau_min    = -bcos - root ;
   length_min =  tau_min*ATM;
   length_max =  tau_max*ATM;
 }
  
////////////////////////////////////////////////////////////////////////
// Returns the 3-D vector of the position on trajectory as a function of tau
////////////////////////////////////////////////////////////////////////
  Hep3Vector NeutrinoTrajectory::R_Tau(double Tau){
// slightly decrease the ATMORADIUS to avoid the round off effects
// in solution of the quadratic equation (just 1 m less) !!
    const double ATM = ATMORADIUS - 1.0; 
    Hep3Vector R_on_Tau;
    if((Tau - tau_min)*(Tau - tau_max) <= 0)
       R_on_Tau =  Rcyl + dn*Tau*ATM;
    else {
       cerr << " Sorry, but parameter tau =" << Tau << " is out of limits " << endl;
       exit(EXIT_FAILURE);
    }
    return R_on_Tau;
  }

////////////////////////////////////////////////////////////////////////
// Returns the 3-D vector of the position in the Earth as a function of Lengt in meters
////////////////////////////////////////////////////////////////////////
  Hep3Vector NeutrinoTrajectory::R_Length(double Length){     
    return R_Tau(Length/ATMORADIUS);
  }

////////////////////////////////////////////////////////////////////////
// End of NeutrTraject

