////////////////////////////////////////////////////////////////////////
#include <Propagation.hh>
////////////////////////////////////////////////////////////////////////

 extern SigmaTotal *sig_tot;
 extern IsNeutrino is_neutrino;
 extern IsUnStable is_unstable;
 extern Earth earth;

////////////////////////////////////////////////////////////////////////
  void Propagate::ThroughEarth(AnisEvent* evt){

// loop over all particles and propagate or let them decay
    //    TauDecay decay;
     for ( HepMC::GenEvent::particle_iterator p = evt->particles_begin();
	 p != evt->particles_end(); ++p ) {
       if (is_neutrino(*p)) { 
         PropagateNeutrino(*p,evt);
       }
       if (is_unstable(*p)){ 
         decay.AddVertex(*p,evt);
       }
     }
 }

 void Propagate::PropagateNeutrino(HepMC::GenParticle* neutrino, AnisEvent* evt){
//  Hep3Vector direction = evt->FinalVolume.GetUnitDirection();

//  end for propagation
    Hep3Vector end_prop  = evt->FinalVolume.GetExitIntersection();
// the position of the start vertex
    Hep3Vector start_vertex;
    start_vertex.setX(neutrino->production_vertex()->position().x());
    start_vertex.setY(neutrino->production_vertex()->position().y());
    start_vertex.setZ(neutrino->production_vertex()->position().z());


    //don't do the propagation if the vertex is behind the cylinder
    int do_prop=1;
    if ((evt->FinalVolume.IsBehindVolume(start_vertex))) 
      do_prop=0;
    
// normalization for interaction crosssection
    double norm = 1e-25 / PMASS;
// local vertex while propagation
    Hep3Vector local_vertex;

// init the propagation algorithm
    int no_interaction = 1;     //init the no-interaction flag
    int nstep          = 1000;   //number of steps
    int istep          = 0;
    Hep3Vector dx      = (end_prop - start_vertex)/nstep;
    double dx_length   = dx.mag();
        
// while not an interaction and no propagation end reached
    while (no_interaction && istep<nstep && do_prop){
      local_vertex  = start_vertex+dx*istep;
      double density     = earth.GetDensity(local_vertex);
      double material[3] = {0.5,0.5,0.5};
      double sigtot      = sig_tot->GetSigma(neutrino,material);

      double p_interaction = 1.0 - exp(-(norm*density*sigtot*dx_length));
      double p_random = drand48();
      if (p_interaction > p_random){  //its an interaction 
        no_interaction = 0;
        double time    = 0; //time of the vertex    
	// choose a random position in the dx interval
	local_vertex+=drand48()*dx;
// create new vertex:
        GenVertex* vertex = new GenVertex(
	     HepLorentzVector(local_vertex,time),1,1);
// add incoming neutrino
        vertex->add_particle_in(neutrino); 
        sig_tot->FillVertex(vertex,material); 
// add vertex
        evt->add_vertex(vertex);
      }
      istep++; //increase steping counter
    }
 }



