////////////////////////////////////////////////////////////////////////
#include <Steer.hh>


////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
 Steer::Steer(char *filename){
 
   char buffer[MAXLINE];
   Sigmafile      = new char*[MAXSIGMA]; 
   Finalfile      = new char*[MAXSIGMA];
   ActiveMaterial = new int  [MAXSIGMA];
   ActiveFlavor   = new int  [MAXSIGMA];
   Token          = new char*[MAXSIGMA];
   Isigma         = 0;             // initialization
   FinalDecay   = 1;               // initialization 
   char* data=getenv("ANIS_DATA");
   if (data!=NULL){
     (void)strcpy(DataDir,data);
     cerr<<"using the eviroment variable ANIS_DATA as data directory" <<endl;
   }

   ifstream steerfile(filename);   // opening input file
   if (!steerfile){
     cerr << " Failed to open steering file " << filename << endl;
      exit(EXIT_FAILURE);
   }
   while(buffer[0]=='#') {
     steerfile.getline(buffer,MAXLINE,'\n');
   } 
   while (1){  
       steerfile.getline(buffer,MAXLINE,' ');  //reading in  option
       if(steerfile.eof()){
	   cerr << " Finished reading steerfile: " << filename << endl;
	   break;
       } else if (buffer[0]=='s'){   // it's a cross section 
	   Sigmafile[Isigma] = new char[MAXLINE];
	   Finalfile[Isigma] = new char[MAXLINE];
	   Token    [Isigma] = new char[MAXLINE];  
	   steerfile.getline(Token[Isigma],MAXLINE,' ');  
	   steerfile.getline(Sigmafile[Isigma],MAXLINE,' ');  
	   steerfile.getline(Finalfile[Isigma],MAXLINE,' ');  
	   steerfile.getline(buffer,MAXLINE,' ');
	   ActiveFlavor[Isigma]=atoi(buffer);
	   steerfile.getline(buffer,MAXLINE,'\n');
	   ActiveMaterial[Isigma]=atoi(buffer);
	   Isigma++;
       } else if (buffer[0]=='d'){   // it's the data dir
	   steerfile.getline(DataDir,MAXDIR,'\n');
	   if (data!=NULL){
	     cerr<<"overwritting the data directory with steering file"<<endl;
	   }
       } else if (buffer[0]=='f'){   // it's the flux line
	   steerfile.getline(FluxType,MAXLINE,' ');  
	   steerfile.getline(buffer,MAXLINE,' ');
	   FluxGamma=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   FluxEmin=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   FluxEmax=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   FluxCosmin=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   FluxCosmax=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,'\n');
	   InitialFlavor=atoi(buffer);
       } else if (buffer[0]=='g'){   // it's the geometry line
	   steerfile.getline(DetectorType,MAXLINE,' ');  
	   steerfile.getline(buffer,MAXLINE,' ');
	   Rmax=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   HBCylinder=atof(buffer);
	   steerfile.getline(buffer,MAXLINE,' ');
	   HACylinder=atof(buffer);

	   steerfile.getline(buffer,MAXLINE,'\n');
	   istringstream buf_str(buffer);
	   double zdepth;
	   buf_str>>zdepth;
	   DetectorCenter.setZ(EARTHRADIUS-zdepth);
	   if (buf_str.eof()) ExtendedVolume=0;
	   else buf_str>>ExtendedVolume;
	   cerr<<"****************************************"<<endl;
	   cerr<<"Extended="<<ExtendedVolume<<endl;
	   cerr<<"****************************************"<<endl;
       } else if (buffer[0]=='r'){   // it's the run line
	   steerfile.getline(buffer,MAXLINE,' ');
	   NumberOfEvents=atoi(buffer);
	   steerfile.getline(buffer,MAXLINE,'\n');
	   istringstream buf_str(buffer);
	   buf_str>>Format;
	   buf_str>>UseWght;
	   buf_str>>Seed;                       // take fixed seed 
	   if (buf_str.eof()) Seed=time(NULL);  // throw random seed
       } else if (buffer[0]=='e'){   // it's the extra command line
	   steerfile.getline(buffer,MAXLINE,'\n');
	   istringstream buf_str(buffer);
	   buf_str>>FinalDecay;
       } else if (buffer[0]=='#'){   // it's a comment
	   steerfile.getline(buffer,MAXLINE,'\n');
       }
   } 
// do some testing
   if (FluxCosmax<FluxCosmin || 1<FluxCosmax || FluxCosmin <-1){
     cerr << " Warning: check cosmax and cosmin again !!!" << endl;
     exit(EXIT_FAILURE); 
   } else if (FluxEmax<FluxEmin) {
     cerr << " Warning: check energy_max and energy_min again !!!" << endl;
     exit(EXIT_FAILURE);
   } else if  (InitialFlavor!=12 && InitialFlavor!=14 &&  
	       InitialFlavor!=16 ) {
     cerr << " Warning: check Initial flavor (must be 12,14,16) !!!" << endl;
     exit(EXIT_FAILURE);
   }     
   steerfile.close();
   if (DataDir==NULL)
     {
       cerr<<"please set enviroment variable ANIS_DATA"<<endl;
       exit(EXIT_FAILURE);
     }


   


}

////////////////////////////////////////////////////////////////////////
 ostream& operator<<(ostream& os, const Steer& s){  
    os << " Emin and Emax: "     << s.FluxEmin 
       << " " << s.FluxEmax<<endl;
    os << " Radius and Height: " << s.Rmax     
       <<" " << s.HBCylinder << " " << s.HACylinder << endl;
    return os;
 }
////////////////////////////////////////////////////////////////////////




