////////////////////////////////////////////////////////////////////////
// Class to read the steering file and fill parameters
////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
#include <sstream>

#include <cstdlib>
#include <ctime>
#include <CLHEP/Vector/ThreeVector.h>
#include <AnisGlobal.hh>

using namespace std;

#define MAXLINE  100             //max number of characters per file name
#define MAXDIR  1000             //max number of characters per data dir
#define MAXSIGMA 100             //max number of different crossections


#ifndef STEER
#define STEER
 
////////////////////////////////////////////////////////////////////////
 class Steer {
   public:
     Steer(char* filename);
     int    Verbose;

// all the run info
     int    NumberOfEvents;
     int    Format;
     int    UseWght;
     long   Seed;
// all the cross section stuff:
     int    Isigma;              // number of crossections
     char **Token;
     char **Sigmafile;
     char **Finalfile;
     int   *ActiveMaterial;
     int   *ActiveFlavor;
// the data directory
     char DataDir[MAXDIR];
//all the flux stuff
     char FluxType[MAXLINE];     // fluxtype (p=power spectrum,..)
     double FluxGamma;           // the gamma of the flux spectrum 
                                 // (or used as some other variable)
     double FluxEmin;            // minimum neutrino energy
     double FluxEmax;            // maximum neutrino energy
     double FluxCosmin;          // minimum incident cos(theta)
     double FluxCosmax;          // maximum incident cos(theta)
     int    InitialFlavor;

// the extras
   bool FinalDecay;
// the geometry stuff:
     char DetectorType[MAXLINE]; //  c=cylinder,..
     double Rmax;                // radius of generation plane  
     double HBCylinder;          // height of cylinder before the center of detector plane
     double HACylinder;          // height of cylinder after the center of detector plane
     int ExtendedVolume;         // if <>0: Cylinder will be extended by an energy dependent lepton range

// center of detector
     Hep3Vector DetectorCenter;
 };

////////////////////////////////////////////////////////////////////////

#endif 



