/////////////////////////////////////////////////////////////////////////////
#include <Decay.hh>
#include <Steer.hh>
/////////////////////////////////////////////////////////////////////////////


  extern Steer* steerdata;
  extern Earth earth;                                  


void ParticleDecay::AddVertex(GenParticle* parent, AnisEvent* evt){
  if  (abs(parent->pdg_id())==15){    // its a tau decay
    taudecay.AddVertex(parent,evt);
  }
}

TauDecay::TauDecay(){
    //  create path to tau decay table 
    char  taudata[1000];
    char *file   = steerdata->DataDir;
    (void)strcpy(taudata,file);
    char *dummy2 = strcat(taudata,"/decay/tau_decay_tauola.dat");
   
    //  init tau decay table 
    int InitTau = TauData.InitTable(taudata);                              
}
/////////////////////////////////////////////////////////////////////////////
// Beta(E) = (1/E)*(dE/dX) in cm^2/g
// 8e-9                       is due to bremstrahlung
// 1.5e-7                     is due to pair-production
// 6.3*10^{-7}*(E/10^9)^(0.2) is due to photonuclear scattering
  double TauDecay::beta( const double E) {

  if ( E > 1E4)  //old parametrization
    return (1.508 + 6.3*pow((E/1e9),0.2))*1e-7;
  else   
    return 2.138e-7;


//      return (0.16 + 0.069*log10(E))*1e-6; // from hep-ph/0308042
  }

void TauDecay::TauProp(HepMC::GenParticle* parent, double &efinal, double &tau_path){
    double lifetime = -1e2*TAU_CTE*log(drand48());   // in cm/GeV
    double ebegin=parent->momentum().e();
    HepLorentzVector vtx_position  = parent->production_vertex()->position();
    double rho = earth.GetDensity(vtx_position.v()); // density of the earth at tau production
    const float rho_min=0.0001;
    
    if (rho<rho_min){
      cerr<<"warning: small rho. Probably something wrong in Decay.cc"<<endl;
      rho=rho_min;
    }
    double bt=beta(ebegin);
    double dt; //lifetime during step
    double efrac=1/pow(10,0.2); //energy fraction after step 
    
    double ebefore = ebegin;
    bt = beta(ebefore);
    dt = 1/rho/bt/ebefore/(1/efrac-1);

    tau_path=0;

    while (dt<lifetime) { // propagate until decay
	dt += 1/rho/bt/ebefore/(1/efrac-1);
	tau_path += -log(efrac)/(rho*bt*100);
	ebefore=ebefore*efrac;
	bt = beta(ebefore);
    }
    double tremain = lifetime-dt+1/rho/bt/ebefore/(1/efrac-1); // the remaining lifetime

    efinal = 1/(1/ebefore + tremain*rho*bt);
    tau_path += -log(efinal/ebefore)/(rho*bt*100);

}

/////////////////////////////////////////////////////////////////////////////
 void TauDecay::AddVertex(GenParticle* parent, AnisEvent* evt){
      HepLorentzVector range;
//throw distance
      double efinal;
      double tau_path;

      if  (abs(parent->pdg_id())==15){    // its a tau decay
	TauProp(parent, efinal, tau_path);
	//create decay vertex
	range.setV( parent->momentum().v().unit()*tau_path);
	range.setT(tau_path/C_LIGHT);
	HepLorentzVector vtx_position  = parent->production_vertex()
	  ->position();
	vtx_position+=range; // change the position of tau
	GenVertex* vtx = new GenVertex(vtx_position,1,1); 
	//add incoming particle
	vtx->add_particle_in(parent);
	double eratio=efinal/parent->momentum().e();
	// change the 4-momentum of tau to the final position
	parent->set_momentum(eratio* parent->momentum());  
	//fill particles from tau decay
	AddTauVertex(parent,vtx);
	// change the 4-momentum of tau back to the original 
	parent->set_momentum(1/eratio* parent->momentum());  

	//add vertex to event
	evt->add_vertex(vtx);
      }
 }

/////////////////////////////////////////////////////////////////////////////
 void TauDecay::AddTauVertex(GenParticle* parent,GenVertex* vtx){
   double finalstate[6];             //nu_tau,nu_mu,nu_e,hadron,muon,electron
   TauData.ThrowFinal(finalstate);

   HepLorentzVector tau_momentum = parent->momentum();
   int sign = parent->pdg_id()/abs(parent->pdg_id());

      GenParticle* tauneutrino = new GenParticle(finalstate[0]*tau_momentum,sign*16,1);
      vtx->add_particle_out(tauneutrino);           // tau neutrino
      if(finalstate[1]!=0.0) {
        GenParticle* muonantineutrino  = new GenParticle(finalstate[1]*tau_momentum,-14*sign,1);
        vtx->add_particle_out(muonantineutrino);       // muon antineutrino
      }
     if(finalstate[2]!=0.0){
      GenParticle* electronantineutrino= new GenParticle(finalstate[2]*tau_momentum,-12*sign,1);
      vtx->add_particle_out(electronantineutrino);   // electron antineutrino
     }
     if(finalstate[3]!=0.0){
      GenParticle* pion = new GenParticle(finalstate[3]*tau_momentum,-211*sign,1);
      vtx->add_particle_out(pion);   // hadron cascade (pions, kaons)
     }
     if(finalstate[4]!=0.0){
      GenParticle* electron = new GenParticle(finalstate[4]*tau_momentum,11*sign,1);
      vtx->add_particle_out(electron);               // em cascade  (electrons, photons)
     }
     if(finalstate[5]!=0.0){
      GenParticle* muon = new GenParticle(finalstate[5]*tau_momentum,13*sign,1);
      vtx->add_particle_out(muon);                   // muons
     }
 }

/////////////////////////////////////////////////////////////////////////////



















