////////////////////////////////////////////////////////////////////////////
// ProcessManager is managing the processes.
// It stores which processes for which flavor for which material.
////////////////////////////////////////////////////////////////////////////
#include <ProcessManager.hh>
#include <string.h>
#include <stdio.h>
#define NOTTHERE "nada"

////////////////////////////////////////////////////////////////////////////
 ProcessManager::ProcessManager(){ 
    Token      = new char*[MAXSIGMA];
    Sigmafile  = new char*[MAXSIGMA];
    Finalfile  = new char*[MAXSIGMA];
    Nsigma=0;        // zero processes loaded at initialization
 }

////////////////////////////////////////////////////////////////////////////
 ProcessManager::ProcessManager(Steer steerdata){ 
   Nsigma    = 0;
   Token     = new char*[MAXSIGMA];
   Sigmafile = new char*[MAXSIGMA];
   Finalfile = new char*[MAXSIGMA];

   char *DataPath = steerdata.DataDir;
   int   plength  = strlen(DataPath);

   for (int i=0; i<steerdata.Isigma; i++){
      int tlength  = strlen(steerdata.Token[i]);
      Token[i]     = new char[tlength+1];
      Token[i]     = steerdata.Token[i];
// creating file name for cross-saction data
      char  s_tmp[1000];
      char  f_tmp[1000];


      strcpy(s_tmp,DataPath);
      strcat(s_tmp,"/");
      strcat(s_tmp,steerdata.Sigmafile[i]);
      Sigmafile[i] = new char[strlen(s_tmp)+1];
      strcpy(Sigmafile[i],s_tmp);

      strcpy(f_tmp,DataPath);
      strcat(f_tmp,"/");
      strcat(f_tmp,steerdata.Finalfile[i]);
      Finalfile[i] = new char[strlen(f_tmp)+1];
      strcpy(Finalfile[i],f_tmp); 

      ActiveFlavor  [i] = steerdata.ActiveFlavor  [i];
      ActiveMaterial[i] = steerdata.ActiveMaterial[i];
      cerr << " loaded process manager with: " << Token[i] <<endl;
      Nsigma++;
   }
 }

////////////////////////////////////////////////////////////////////////////
 ProcessManager::~ProcessManager() {  // deleting everything
    for (int i=0; i<Nsigma; i++){
      delete  Token[i];
      delete  Sigmafile[i];
      delete  Finalfile[i];
    }
    delete[] Token;
    delete[] Finalfile;
    delete[] Sigmafile;
 }
 
////////////////////////////////////////////////////////////////////////////
 void ProcessManager::AddProcess(char *newtoken, char *filename, 
      char *finalfile, int flavor, double material[]) {
    int filelength  = strlen(filename);
    int finallength = strlen(finalfile);
    int tokenlength = strlen(newtoken);

    Sigmafile[Nsigma] = new char[filelength+1];
    Finalfile[Nsigma] = new char[finallength+1];
    Token    [Nsigma] = new char[tokenlength+1]; 
    strcpy(Sigmafile[Nsigma],filename);
    strcpy(Token[Nsigma],newtoken) ;
    strcpy(Finalfile[Nsigma],finalfile) ;
    cerr << "ProcessManager: Added " << Token[Nsigma] << endl;
    Nsigma++;
 }  

////////////////////////////////////////////////////////////////////////////
 char* ProcessManager::GetToken(int sigma_i){
    if (sigma_i<Nsigma) {   // if sigma_i registered
      return Token[sigma_i];
    } else {
    return NOTTHERE;
    }
 }
 
////////////////////////////////////////////////////////////////////////////
 char* ProcessManager::GetSigmaFile(int sigma_i){
   if (sigma_i<Nsigma) {   // if sigma_i registered
      return Sigmafile[sigma_i];
   } else {
      return NOTTHERE;
   }
 }

////////////////////////////////////////////////////////////////////////////
 char* ProcessManager::GetFinalFile(int sigma_i){
   if (sigma_i<Nsigma) { // if sigma_i registered
     return Finalfile[sigma_i];
   } else {
     return NOTTHERE;
   }
 }
 
////////////////////////////////////////////////////////////////////////////
 int ProcessManager::IsFlavorActive(int sigma_i, int flavor){
    int active;
// explanation:
// the sheme is nue, nuebar, numu, numubar, nutau, nutaubar
// example: 1001 means numu and nutaubar are active 
    if      (ActiveFlavor[sigma_i] == 101010 && flavor  >   0 )  active = 1;   // its a  neutrino
    else if (ActiveFlavor[sigma_i] ==  10101 && flavor  <   0 )  active = 1;   // its an anti neutrino
    else if (ActiveFlavor[sigma_i] ==  10000 &&	flavor == -12 )  active = 1;   // its an anti electron neutrino
    else if (ActiveFlavor[sigma_i] == 100000 && flavor ==  12 )  active = 1;   // its an electron neutrino
    else if (ActiveFlavor[sigma_i] ==    100 && flavor == -14 )  active = 1;   // its an anti muon neutrino
    else if (ActiveFlavor[sigma_i] ==   1000 && flavor ==  14 )  active = 1;   // its a  muon neutrino
    else if (ActiveFlavor[sigma_i] ==      1 && flavor == -16 )  active = 1;   // its an anti tau  neutrino
    else if (ActiveFlavor[sigma_i] ==     10 && flavor ==  16 )  active = 1;   // its a  tau neutrino
    else active = 0;
    return active;
 } 

////////////////////////////////////////////////////////////////////////////
 int ProcessManager::IsMaterialActive(int sigma_i, int material){
    int active;
// proton, neutron, electron
    if      ((ActiveMaterial[sigma_i] ==  10 || ActiveMaterial[sigma_i] ==  11 ||
	      ActiveMaterial[sigma_i] == 111 || ActiveMaterial[sigma_i] == 110) 
	 &&  (material ==   PROTON) ) active = 1;      // proton active
    else if ((ActiveMaterial[sigma_i] == 100 || ActiveMaterial[sigma_i] == 110 ||
	      ActiveMaterial[sigma_i] == 101 || ActiveMaterial[sigma_i] == 111) 
	 &&  (material ==  NEUTRON) ) active = 1;      // neutron active	
    else if ((ActiveMaterial[sigma_i] == 111 || ActiveMaterial[sigma_i] == 101 ||
	      ActiveMaterial[sigma_i] ==   1 || ActiveMaterial[sigma_i] ==  11) 
	 &&  (material == ELECTRON) ) active = 1;      //electron active
    else active = 0;
    return active;
}

////////////////////////////////////////////////////////////////////////////
    










