////////////////////////////////////////////////////////////////
// SigmaTotal is the class containing all active interaction channels
////////////////////////////////////////////////////////////////

#include<SigmaTotal.hh>

////////////////////////////////////////////////////////////////

 extern Steer *steerdata;   
 extern ProcessManager *prmanager;

////////////////////////////////////////////////////////////////
 SigmaTotal::SigmaTotal() {  // initializing the cross sections
   SigmaInclusive = new Sigma*[steerdata->Isigma];
   for(int i=0; i<prmanager->HowManyProcesses(); i++){ 
       if       (!strcmp(prmanager->GetToken(i),"NC")){  // it is a NC  reaction
	  SigmaNC* sigmatmp = new SigmaNC;
	  SigmaInclusive[i] = sigmatmp;
      } else if (!strcmp(prmanager->GetToken(i),"CC")){  // it is a CC  reaction
	  SigmaCC* sigmatmp = new SigmaCC;
	  SigmaInclusive[i] = sigmatmp;
      } else if (!strcmp(prmanager->GetToken(i),"GR")){  // Glashow resonance
	  SigmaGR* sigmatmp = new SigmaGR;
	  SigmaInclusive[i] = sigmatmp;
      } else {
	  cerr << prmanager->GetToken(i) <<
	      " is not a defined cross section"<<endl;       
      }
      SigmaInclusive[i]->InitSigma ( prmanager->GetToken(i),
          prmanager->GetSigmaFile(i), prmanager->GetFinalFile(i));
   }
 }

////////////////////////////////////////////////////////////////
// GetSigma returns sum of inclusive cross-sections 
 double SigmaTotal::GetSigma(GenParticle* nuin, double material[]){
    double total=0.;
    for(int i=0; i < steerdata->Isigma; i++) {
       if (prmanager->IsFlavorActive(i,nuin->pdg_id())){
	 if ( prmanager->IsMaterialActive(i,PROTON)) { // proton
	      total += material[0]*(SigmaInclusive[i]->GetSigma(nuin,material));
	 }
         if ( prmanager->IsMaterialActive(i,NEUTRON)) { //neutron
	      total+=material[1]*(SigmaInclusive[i]->GetSigma(nuin,material));
	 }
	 if ( prmanager->IsMaterialActive(i,ELECTRON)) { //electron
	      total+=material[2]*(SigmaInclusive[i]->GetSigma(nuin,material));
	 }
      }
    }
    return total;
 }

////////////////////////////////////////////////////////////////
// choose inclusive process and return corresponding final state  
 void SigmaTotal::FillVertex(GenVertex* vtx, double material[]) {
    
// there should be only one parent neutrino in vertex 
      GenVertex::particle_iterator nuin = 
         vtx->particles_begin(HepMC::parents);   
      double rand = drand48() * GetSigma(*nuin,material);    
      double  sum =  0;
      int       i = -1;                // initializing index of final state 

      while (rand > 0 && rand > sum) { // choosing index of final state 
	  i++;
	  if (prmanager->IsFlavorActive(i,(*nuin)->pdg_id())){
	      if (prmanager->IsMaterialActive(i,PROTON))    // proton
		  sum += material[0]*(SigmaInclusive[i]->GetSigma(*nuin,material));
	      if (prmanager->IsMaterialActive(i,NEUTRON))   // neutron
		  sum += material[1]*(SigmaInclusive[i]->GetSigma(*nuin,material));
	      if (prmanager->IsMaterialActive(i,ELECTRON))  // electron
		  sum += material[2]*(SigmaInclusive[i]->GetSigma(*nuin,material));
	  }
      }
      if (!(rand==0)) {               // if active neutrino
	  SigmaInclusive[i]->FillVertex(vtx, material);  
	  vtx->set_id(i);
      }
 }    

////////////////////////////////////////////////////////////////
// destructor is responsible for deleting the partial cross sections
 SigmaTotal::~SigmaTotal(){
    for (int i=0; i < prmanager->HowManyProcesses(); i++)
	delete[] SigmaInclusive[i];
    delete[] SigmaInclusive;
 }

////////////////////////////////////////////////////////////////




